@@ -13,9 +13,13 @@
 //        swgames  -      SW definition of games
 //
 
+#include <assert.h>
+#include <string.h>
 
 #include "sw.h"
 #include "swgames.h"
+#include "video.h"
+#include "yocton.h"
 
 static GRNDTYPE original_ground[] = { /* Original ground height by pixel */
 
@@ -211,49 +215,50 @@ static GRNDTYPE original_ground[] = { /* Original ground height by pixel */
 
 
 static original_ob_t original_targets[] = {
-	{TARGET,  191, 1, 0, 0, OWNER_PLAYER1_IN_MULT},
-	{TARGET,  284, 3, 0, 0, OWNER_PLAYER1_IN_MULT},
-	{TARGET,  409, 1, 0, 0, OWNER_PLAYER1_IN_MULT},
-	{TARGET,  539, 1, 0, 0, OWNER_PLAYER1_IN_MULT},
-	{TARGET,  685, 3, 0, 0, OWNER_PLAYER1_IN_MULT},
-	{TARGET,  807, 0, 0, 0, OWNER_PLAYER1_IN_MULT},
-	{TARGET,  934, 1, 0, 0, OWNER_PLAYER1_IN_MULT},
+	// Left side of map -
+	{TARGET,  191, 1, 0, 0, OWNER_PLAYER3},
+	{TARGET,  284, 3, 0, 0, OWNER_PLAYER3},
+	{FLOCK,   370, 0, 370, 2630, OWNER_NONE},
+	{TARGET,  409, 1, 0, 0, OWNER_PLAYER3},
+	{TARGET,  539, 1, 0, 0, OWNER_PLAYER3},
+	{PLANE,   588, 0, 0, 1154, OWNER_PLAYER3},
+	{TARGET,  685, 3, 0, 0, OWNER_PLAYER3},
+	{TARGET,  807, 0, 0, 0, OWNER_PLAYER3},
+	{TARGET,  934, 1, 0, 0, OWNER_PLAYER3},
 
+	// Player 1 base
+	{FLOCK,  1000, 0, 370, 2630, OWNER_NONE},
 	{TARGET, 1210, 2, 0, 0, OWNER_PLAYER1},
 	{TARGET, 1240, 0, 0, 0, OWNER_PLAYER1},
-	{OX,     1376, 0, 0, 0, OWNER_PLAYER1},
+	{PLANE,  1270, 0,  901, 1835, OWNER_PLAYER1},
+	{PLANE,  1330, 0,  901, 1835, OWNER_PLAYER5}, // Extra planes for multiplayer
+	{PLANE,  1360, 0,  901, 1835, OWNER_PLAYER6},
+	{OX,     1376, 0, 0, 0, OWNER_NONE},
 	{TARGET, 1440, 3, 0, 0, OWNER_PLAYER1},
 
+	// Player 2 base
 	{TARGET, 1550, 3, 0, 0, OWNER_PLAYER2},
-	{OX,     1608, 0, 0, 0, OWNER_PLAYER1},
+	{OX,     1608, 0, 0, 0, OWNER_NONE},
+	{FLOCK,  1630, 0, 370, 2630, OWNER_NONE},
+	{PLANE,  1630, 1, 1155, 2089, OWNER_PLAYER7}, // Extra planes for multiplayer
+	{PLANE,  1660, 1, 1155, 2089, OWNER_PLAYER8},
+	{PLANE,  1720, 1, 1155, 2089, OWNER_PLAYER2},
 	{TARGET, 1750, 0, 0, 0, OWNER_PLAYER2},
 	{TARGET, 1780, 2, 0, 0, OWNER_PLAYER2},
 	{TARGET, 2024, 1, 0, 0, OWNER_PLAYER2},
-	{TARGET, 2159, 1, 0, 0, OWNER_PLAYER2},
-	{TARGET, 2279, 3, 0, 0, OWNER_PLAYER2},
-	{TARGET, 2390, 3, 0, 0, OWNER_PLAYER2},
-	{TARGET, 2549, 0, 0, 0, OWNER_PLAYER2},
-	{TARGET, 2678, 0, 0, 0, OWNER_PLAYER2},
-	{TARGET, 2763, 1, 0, 0, OWNER_PLAYER2},
 
-	{FLOCK,   370, 0, 370, 2630},
-	{FLOCK,  1000, 0, 370, 2630},
-	{FLOCK,  1630, 0, 370, 2630},
-	{FLOCK,  2630, 0, 370, 2630},
+	// Right side of map -
+	{TARGET, 2159, 1, 0, 0, OWNER_PLAYER4},
+	{TARGET, 2279, 3, 0, 0, OWNER_PLAYER4},
+	{TARGET, 2390, 3, 0, 0, OWNER_PLAYER4},
+	{PLANE,  2456, 1, 2089, 9999, OWNER_PLAYER4},
+	{TARGET, 2549, 0, 0, 0, OWNER_PLAYER4},
+	{FLOCK,  2630, 0, 370, 2630, OWNER_NONE},
+	{TARGET, 2678, 0, 0, 0, OWNER_PLAYER4},
+	{TARGET, 2763, 1, 0, 0, OWNER_PLAYER4},
 };
 
-GAMES original_level = {
-	{
-		{PLANE, 1270, 0,  901, 1835},
-		{PLANE, 1720, 1, 1155, 2089},
-		{PLANE,  588, 0,    0, 1154},
-		{PLANE, 2456, 1, 2089, 9999},
-		// Extra planes for multiplayer:
-		{PLANE, 1330, 0,  901, 1835},
-		{PLANE, 1360, 0,  901, 1835},
-		{PLANE, 1630, 1, 1155, 2089},
-		{PLANE, 1660, 1, 1155, 2089},
-	},
+const GAMES original_level = {
 	7491,
 	original_targets,
 	arrlen(original_targets),
@@ -261,6 +266,118 @@ GAMES original_level = {
 	arrlen(original_ground),
 };
 
+static const char *obtype_names[] = {
+	"GROUND", "PLANE", "BOMB", "SHOT", "TARGET", "EXPLOSION", "SMOKE",
+	"FLOCK", "BIRD", "OX", "MISSILE", "STARBURST", NULL,
+};
+
+static const char *owner_names[] = {
+	"NONE", "PLAYER1", "PLAYER2", "PLAYER3", "PLAYER4", "PLAYER5",
+	"PLAYER6", "PLAYER7", "PLAYER8", NULL,
+};
+
+GAMES custom_level;
+bool have_custom_level;
+
+#define cl custom_level
+
+static void free_custom_level(void)
+{
+	free(cl.gm_objects);
+	cl.gm_objects = NULL;
+	cl.gm_num_objects = 0;
+	free(cl.gm_ground);
+	cl.gm_ground = NULL;
+	cl.gm_max_x = 0;
+	cl.gm_rseed = 12345;
+}
+
+static void add_object(original_ob_t *ob, struct yocton_object *yo)
+{
+	struct yocton_prop *p;
+
+	while ((p = yocton_next_prop(yo)) != NULL) {
+		YOCTON_VAR_INT(p, x, int, ob->x);
+		YOCTON_VAR_INT(p, orient, int, ob->orient);
+		YOCTON_VAR_INT(p, territory_l, int, ob->territory_l);
+		YOCTON_VAR_INT(p, territory_r, int, ob->territory_r);
+		YOCTON_VAR_ENUM(p, type, ob->type, obtype_names);
+		YOCTON_VAR_ENUM(p, owner, ob->owner, owner_names);
+	}
+}
+
+static void set_ground(struct yocton_object *yo)
+{
+	struct yocton_prop *p;
+
+	while ((p = yocton_next_prop(yo)) != NULL) {
+		yocton_check(yo, "expected prop name '_'",
+		             !strcmp(yocton_prop_name(p), "_"));
+		YOCTON_VAR_INT_ARRAY(p, _, GRNDTYPE, cl.gm_ground,
+		                     cl.gm_max_x);
+	}
+}
+
+static void process_level(struct yocton_object *obj)
+{
+	struct yocton_prop *p;
+
+	while ((p = yocton_next_prop(obj)) != NULL) {
+		YOCTON_VAR_ARRAY(p, object, cl.gm_objects, cl.gm_num_objects, {
+			add_object(&cl.gm_objects[cl.gm_num_objects],
+				yocton_prop_inner(p));
+			++cl.gm_num_objects;
+		});
+		YOCTON_IF_PROP(p, ground, {
+			set_ground(yocton_prop_inner(p));
+		});
+	}
+}
+
+void load_custom_level(const char *filename)
+{
+	FILE *fs;
+	struct yocton_object *obj;
+	const char *error_msg;
+	int lineno;
+	struct yocton_prop *p;
+	bool processed_level = false;
+
+	free_custom_level();
+
+	fs = fopen(filename, "r");
+	if (fs == NULL) {
+		error_exit("Failed to open file: %s", filename);
+	}
+	obj = yocton_read_from(fs);
+	assert(obj != NULL);
+
+	while ((p = yocton_next_prop(obj)) != NULL) {
+		const char *name = yocton_prop_name(p);
+		// TODO: Add support for multiple levels within a mission file.
+		// The level data gets embedded within a level {} object with
+		// the expectation that files will be able to contain multiple
+		// levels in the future.
+		if (!strcmp(name, "level")) {
+			yocton_check(obj, "only one level per file supported",
+			             !processed_level);
+			process_level(yocton_prop_inner(p));
+			processed_level = true;
+		}
+	}
+	yocton_check(obj, "expected level {} block in file", processed_level);
+
+	if (yocton_have_error(obj, &lineno, &error_msg)) {
+		error_exit("Error in %s at line %d:\n%s", filename, lineno,
+		           error_msg);
+	}
+
+	yocton_free(obj);
+	have_custom_level = true;
+}
+
+#undef cl
+
 //
 // 2003-02-14: Code was checked into version control; no further entries
 // will be added to this log.
