@@ -16,6 +16,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
+#include <assert.h>
 
 #include "pcsound.h"
 #include "timer.h"
@@ -35,59 +36,50 @@
 #include "swsymbol.h"
 #include "swtitle.h"
 
+static const original_ob_t *orig_planes[MAX_PLYR * 2];
 GRNDTYPE *ground;
 
 static int have_savescore = 0;
 static score_t savescore;		/* save players score on restart  */
-static int starting_level = 0;
+int starting_level = 0;
 
 static char helptxt[] =
 "\n"
 PACKAGE_STRING "\n"
 "Copyright (C) 1984, 1985, 1987 BMB Compuscience\n"
 "Copyright (C) 1984-2000 David L. Clark\n"
-"Copyright (C) 2001-2022 Simon Howard, Jesse Smith\n"
+"Copyright (C) 2001-2024 Simon Howard, Jesse Smith\n"
 "Licensed under the GNU GPL v2.\n"
 "\n"
 "Usage:  sopwith [options]\n"
 "The options are:\n"
-"        -n :  novice single player\n"
-"        -s :  single player\n"
-"        -c :  single player against computer\n"
-"        -q :  begin game with sound off\n"
-"        -g#:  start at level #\n"
-"        -e :  turn off big explosions\n"
+"        -n : novice single player\n"
+"        -s : single player\n"
+"        -c : single player against computer\n"
+"       -g# : start at level #\n"
 "\n"
-"Video:\n"
-"        -f    :  fullscreen\n"
+"        -e : turn off big explosions\n"
+"        -f : fullscreen\n"
+" -m <file> : load new mission from file\n"
+"        -q : begin game with sound off\n"
 "\n"
 #ifdef TCPIP
 "Networking: \n"
-"        -l    :  listen for connection\n"
-" -j <host>    :  connect to a listening host\n"
+" -j <host> : connect to a listening host\n"
+"        -l : listen for connection\n"
+" -p <port> : use alternative TCP port\n"
 #endif
 ;
 
 static void initobjs(void)
 {
-	OBJECTS *ob;
-	int i;
-
 	topobj.ob_xnext = topobj.ob_next = &botobj;
 	botobj.ob_xprev = botobj.ob_prev = &topobj;
 	topobj.ob_x = -32767;
 	botobj.ob_x = 32767;
 
 	objbot = objtop = deltop = delbot = NULL;
-	objfree = ob = nobjects;
-
-	for (i = 0; i < MAX_OBJS; ++i) {
-		ob->ob_next = ob + 1;
-		ob->ob_index = i;
-		++ob;
-	}
-
-	(ob-1)->ob_next = NULL;
+	objfree = NULL;
 }
 
 
@@ -118,16 +110,14 @@ void initdisp(bool reset)
 // object creation
 //
 
-
-static void initplanescore (score_t *score)
+static void initscore(score_t *score)
 {
-	score->planekills = 0;
-	score->medals = 0x0000;
-	score->valour = 0;
-	score->killscore = 0;
-	score->landings = 0;
-	score->medals_nr = 0;
-	score->ribbons_nr = 0;
+	memset(score, 0, sizeof(score_t));
+}
+
+static void initflightscore(flight_score_t *score)
+{
+	memset(score, 0, sizeof(flight_score_t));
 }
 
 #define SERVICE_KILLSCORE 5
@@ -135,95 +125,97 @@ static void initplanescore (score_t *score)
 #define VALOUR_PRELIMIT 175
 #define VALOUR_LIMIT 250
 
+static bool have_medal(int *medals, int medals_nr, int medal_id)
+{
+	int i;
+
+	for (i = 0; i < medals_nr; i++) {
+		if (medals[i] == medal_id) {
+			return true;
+		}
+	}
+
+	return false;
+}
+
 static void give_medal(OBJECTS *ob, int medal_id)
 {
-	ob->ob_score.medalslist[ob->ob_score.medals_nr++] = medal_id;
+	score_t *sc = &ob->ob_score;
+	if (!have_medal(sc->medals, sc->medals_nr, medal_id)) {
+		assert(sc->medals_nr < 3);
+		sc->medals[sc->medals_nr++] = medal_id;
+	}
 }
 
 static void give_ribbon(OBJECTS *ob, int ribbon_id)
 {
-	ob->ob_score.ribbons[ob->ob_score.ribbons_nr++] = ribbon_id;
+	score_t *sc = &ob->ob_score;
+	if (!have_medal(sc->ribbons, sc->ribbons_nr, ribbon_id)) {
+		assert(sc->ribbons_nr < 6);
+		sc->ribbons[sc->ribbons_nr++] = ribbon_id;
+	}
 }
 
 static void get_awards(OBJECTS *ob)
 {
-	score_t *lsc = &ob->ob_lastscore;
+	flight_score_t *fsc = &ob->ob_flightscore;
 	score_t *sc = &ob->ob_score;
 
-	if (!(lsc->medals & MEDAL_PURPLEHEART)) {
-		// We only award purple heart if wounded in combat (=hit by
-		// bullet), not from a bird strike or flying into an ox
-		if (ob->ob_score.combatwound) {
-			sc->medals |= MEDAL_PURPLEHEART;
-			give_medal(ob, MEDAL_ID_PURPLEHEART);
+	// We only award purple heart if wounded in combat (=hit by bullet),
+	// not from a bird strike or flying into an ox.
+	if (fsc->combatwound) {
+		give_medal(ob, MEDAL_PURPLEHEART);
+	}
+
+	// We count up the number of planes shot down, but they only count if
+	// the player returns to base successfully.
+	sc->planekills += fsc->planekills;
+	//printf("planes %d %d\n", sc->planekills, fsc->planekills);
+	if (sc->planekills >= 5) {
+		give_ribbon(ob, RIBBON_ACE);
+	}
+	if (sc->planekills >= 25) {
+		give_ribbon(ob, RIBBON_TOPACE);
+	}
+
+	// We count the number of landings during which we did a decent amount
+	// of damage to the enemy.
+	if (fsc->killscore >= SERVICE_KILLSCORE) {
+		sc->landings++;
+	}
+	//printf("kills %d landings %d\n", fsc->killscore, sc->landings);
+	if (sc->landings >= 3) {
+		give_ribbon(ob, RIBBON_SERVICE);
+	}
+
+	// If a huge number of damage is done, we give a medal (first time) or
+	// ribbon (second time)
+	if (fsc->killscore >= COMPETENCE_KILLSCORE) {
+		if (!have_medal(sc->medals, sc->medals_nr, MEDAL_COMPETENCE)) {
+			give_medal(ob, MEDAL_COMPETENCE);
+		} else {
+			give_ribbon(ob, RIBBON_COMPETENCE2);
 		}
 	}
 
-	if (!(lsc->medals & MEDAL_ACE)) {
-		if (sc->planekills >= 5) {
-			sc->medals |= MEDAL_ACE;
-			give_ribbon(ob, RIBBON_ID_ACE);
-		}
+	// We count up valour, but again - only if the player returned to base
+	// successfully
+	sc->valour += fsc->valour;
+	//printf("valour %d %d\n", sc->valour, fsc->valour);
+	if (sc->valour >= VALOUR_PRELIMIT) {
+		give_ribbon(ob, RIBBON_PREVALOUR);
 	}
-
-	if (!(lsc->medals & MEDAL_TOPACE)) {
-		if (sc->planekills >= 25) {
-			sc->medals |= MEDAL_TOPACE;
-			give_ribbon(ob, RIBBON_ID_TOPACE);
-		}
+	if (sc->valour >= VALOUR_LIMIT) {
+		give_medal(ob, MEDAL_VALOUR);
 	}
-
-	if (!(lsc->medals & MEDAL_SERVICE)) {
-		if (sc->killscore >= SERVICE_KILLSCORE) {
-			sc->landings++;
-		}
-
-		if (sc->landings >= 3) {
-			sc->medals |= MEDAL_SERVICE;
-			give_ribbon(ob, RIBBON_ID_SERVICE);
-		}
-	}
-
-	if (sc->killscore >= COMPETENCE_KILLSCORE) {
-		if (!(lsc->medals & MEDAL_COMPETENCE)) {
-			sc->medals |= MEDAL_COMPETENCE;
-			give_medal(ob, MEDAL_ID_COMPETENCE);
-		} else if (!(lsc->medals & MEDAL_COMPETENCE2)) {
-			sc->medals |= MEDAL_COMPETENCE2;
-			give_ribbon(ob, RIBBON_ID_COMPETENCE2);
-		}
-	}
-
-	if (!(lsc->medals & MEDAL_PREVALOUR)) {
-		if (sc->valour >= VALOUR_PRELIMIT) {
-			sc->medals |= MEDAL_PREVALOUR;
-			give_ribbon(ob, RIBBON_ID_PREVALOUR);
-		}
-	}
-
-	if (!(lsc->medals & MEDAL_VALOUR)) {
-		if (sc->valour >= VALOUR_LIMIT) {
-			sc->medals |= MEDAL_VALOUR;
-			give_medal(ob, MEDAL_ID_VALOUR);
-		}
-	}
-
-	sc->killscore = 0;
-	ob->ob_lastscore = ob->ob_score;
 }
 
 static void get_endlevel(OBJECTS *ob)
 {
-	score_t *sc = &ob->ob_score;
-	score_t *lsc = &ob->ob_lastscore;
-
 	get_awards(ob);
 
-	if (!(lsc->medals & MEDAL_PERFECT)) {
-		if (ob->ob_crashcnt == 0) {
-			sc->medals |= MEDAL_PERFECT;
-			give_ribbon(ob, RIBBON_ID_PERFECT);
-		}
+	if (ob->ob_crashcnt == 0) {
+		give_ribbon(ob, RIBBON_PERFECT);
 	}
 }
 
@@ -232,22 +224,25 @@ static void get_endlevel(OBJECTS *ob)
 OBJECTS *initpln(OBJECTS * obp)
 {
 	OBJECTS *ob;
-	int x, height, minx, maxx, n;
+	int x, height, minx, maxx;
 
 	if (!obp) {
+		assert(num_planes < MAX_PLYR);
 		ob = allocobj();
+		ob->ob_original_ob = orig_planes[num_planes];
+		planes[num_planes] = ob;
+		++num_planes;
 	} else {
 		ob = obp;
 	}
 
-	n = ob->ob_index;
-	ob->ob_original_ob = &currgame->gm_planes[n];
-
 	if (obp && ob->ob_state != CRASHED && !ob->ob_athome) {
 		/* Just returned home */
 		get_awards(ob);
 	}
 
+	initflightscore(&ob->ob_flightscore);
+
 	ob->ob_type = PLANE;
 
 	ob->ob_x = ob->ob_original_ob->x;
@@ -260,6 +255,8 @@ OBJECTS *initpln(OBJECTS * obp)
 		}
 	}
 	ob->ob_y = height + 13;
+	ob->ob_orig_y = ob->ob_y;
+
 	ob->ob_lx = ob->ob_ly = ob->ob_speed = ob->ob_flaps = ob->ob_accel
 	    = ob->ob_hitcount = ob->ob_bdelay = ob->ob_mdelay
 	    = ob->ob_bsdelay = 0;
@@ -272,7 +269,7 @@ OBJECTS *initpln(OBJECTS * obp)
 	ob->ob_newsym = symbol_plane[ob->ob_orient][0];
 	ob->ob_athome = true;
 	ob->ob_onmap = true;
-	ob->ob_score.combatwound = false;
+	ob->ob_flightscore.combatwound = false;
 
 	if (!obp || ob->ob_state == CRASHED) {
 		/* New plane */
@@ -281,11 +278,9 @@ OBJECTS *initpln(OBJECTS * obp)
 		ob->ob_missiles = MAXMISSILES;
 		ob->ob_bursts = MAXBURSTS;
 		ob->ob_life = MAXFUEL;
-		initplanescore(&ob->ob_lastscore);
 	}
 	if (!obp) {
-		initplanescore(&ob->ob_score);
-		ob->ob_score.score = ob->ob_crashcnt = 0;
+		initscore(&ob->ob_score);
 		ob->ob_endsts = PLAYING;
 		ob->ob_target = NULL;
 		insertx(ob, &topobj);
@@ -312,9 +307,8 @@ void initplyr(OBJECTS * obp)
 	if (!obp) {
 		ob->ob_drawf = dispplyr;
 		ob->ob_movef = moveplyr;
-		ob->ob_clr = ob->ob_index % 2 + 1;
+		ob->ob_clr = ob->ob_original_ob->owner;
 		ob->ob_owner = ob;
-		oobjects[ob->ob_index] = *ob;
 		endcount = 0;
 
 		ob->ob_plrnum = num_players;
@@ -340,14 +334,7 @@ void initcomp(OBJECTS * obp)
 		ob->ob_drawf = dispcomp;
 		ob->ob_movef = movecomp;
 		ob->ob_clr = 2;
-		if (playmode != PLAYMODE_ASYNCH) {
-			ob->ob_owner = &nobjects[1];
-		} else if (ob->ob_index == 1) {
-			ob->ob_owner = ob;
-		} else {
-			ob->ob_owner = ob - 2;
-		}
-		oobjects[ob->ob_index] = *ob;
+		ob->ob_owner = planes[1];
 	}
 	if (playmode == PLAYMODE_SINGLE || playmode == PLAYMODE_NOVICE) {
 		ob->ob_state = FINISHED;
@@ -587,7 +574,7 @@ void initburst(OBJECTS *obo)
 
 // building/target
 
-static OBJECTS *inittarget(original_ob_t *orig_ob)
+static OBJECTS *inittarget(const original_ob_t *orig_ob)
 {
 	OBJECTS *ob;
 	int x;
@@ -622,21 +609,33 @@ static OBJECTS *inittarget(original_ob_t *orig_ob)
 	ob->ob_state = STANDING;
 	ob->ob_orient = orig_ob->orient;
 
+	// TODO: For the time being, player 3 is just interpreted as a
+	// special player number that either maps to player 1 or 2
+	// depending on the game mode; other players are just synonyms
+	// for players 1 and 2.
+	// This may change in the future if we support multiplayer with
+	// more than two players.
 	switch (orig_ob->owner) {
+		case OWNER_NONE:
 		case OWNER_PLAYER1:
-			ob->ob_owner = &nobjects[0];
+		case OWNER_PLAYER5:
+		case OWNER_PLAYER7:
+			ob->ob_owner = planes[0];
 			++numtarg[0];
 			break;
 		case OWNER_PLAYER2:
-			ob->ob_owner = &nobjects[1];
+		case OWNER_PLAYER4:
+		case OWNER_PLAYER6:
+		case OWNER_PLAYER8:
+			ob->ob_owner = planes[1];
 			++numtarg[1];
 			break;
-		case OWNER_PLAYER1_IN_MULT:
+		case OWNER_PLAYER3:
 			if (playmode == PLAYMODE_ASYNCH) {
-				ob->ob_owner = &nobjects[0];
+				ob->ob_owner = planes[0];
 				++numtarg[0];
 			} else {
-				ob->ob_owner = &nobjects[1];
+				ob->ob_owner = planes[1];
 				++numtarg[1];
 			}
 			break;
@@ -758,7 +757,7 @@ void initsmok(OBJECTS *obo)
 
 // birds
 
-static OBJECTS *initflock(original_ob_t *orig_ob)
+static OBJECTS *initflock(const original_ob_t *orig_ob)
 {
 	OBJECTS *ob;
 	int j;
@@ -784,7 +783,7 @@ static OBJECTS *initflock(original_ob_t *orig_ob)
 	ob->ob_newsym = symbol_flock[0];
 	ob->ob_drawf = NULL;
 	ob->ob_movef = moveflck;
-	ob->ob_clr = 9;
+	ob->ob_clr = 1;
 	ob->ob_onmap = true;
 
 	for (j = 0; j < NUM_STRAY_BIRDS; ++j) {
@@ -828,7 +827,7 @@ void initbird(OBJECTS *obo, int i)
 
 // oxen
 
-static OBJECTS *initox(original_ob_t *orig_ob)
+static OBJECTS *initox(const original_ob_t *orig_ob)
 {
 	OBJECTS *ob;
 
@@ -858,14 +857,14 @@ static OBJECTS *initox(original_ob_t *orig_ob)
 static void inittargets(void)
 {
 	OBJECTS *ob;
-	original_ob_t *orig_ob;
+	const original_ob_t *orig_ob;
 	int i;
 
 	numtarg[0] = 0;
 	numtarg[1] = 0;
 
-	for (i = 0; i < currgame->gm_num_targets; i++) {
-		orig_ob = &currgame->gm_targets[i];
+	for (i = 0; i < currgame->gm_num_objects; i++) {
+		orig_ob = &currgame->gm_objects[i];
 		switch (orig_ob->type) {
 			case TARGET:
 				ob = inittarget(orig_ob);
@@ -903,6 +902,12 @@ void swinitlevel(void)
 {
 	int i;
 
+	if (have_custom_level) {
+		currgame = &custom_level;
+	} else {
+		currgame = &original_level;
+	}
+
 	// clear out any waiting keys. this stops, eg. 's' on the
 	// menu from toggling sound once the game starts
 
@@ -919,6 +924,20 @@ void swinitlevel(void)
 	initobjs();
 
 	num_players = 0;
+	num_planes = 0;
+	memset(planes, 0, sizeof(planes));
+	memset(orig_planes, 0, sizeof(orig_planes));
+
+	for (i = 0; i < currgame->gm_num_objects; i++) {
+		int pln_index;
+		if (currgame->gm_objects[i].type != PLANE) {
+			continue;
+		}
+		pln_index = currgame->gm_objects[i].owner - 1;
+		if (pln_index >= 0 && pln_index < MAX_PLYR * 2) {
+			orig_planes[pln_index] = &currgame->gm_objects[i];
+		}
+	}
 
 	if (keydelay == -1) {
 		keydelay = 1;
@@ -930,17 +949,19 @@ void swinitlevel(void)
 	} else {
 		maxcrash = MAXCRASH;
 
-		// single player
-		
+		// single player. we spawn as many enemy planes as we have
+		// defined within the map.
+		// TODO: We should have a way to define the number of enemy
+		// planes in levels.
 		initplyr(NULL);
-		initcomp(NULL);
-		initcomp(NULL);
-		initcomp(NULL);
+		for (i = 1; i < MAX_PLYR && orig_planes[i] != NULL; i++) {
+			initcomp(NULL);
+		}
 	}
 
 	inittargets();
 
-	initdisp(NO);
+	initdisp(false);
 	initgdep();
 
 	// sdh 16/11/2001: this needs to be reset with each new game
@@ -961,7 +982,7 @@ void swrestart(void)
 	int time;
 		
 	if (consoleplayer->ob_endsts == WINNER) {
-		ob = &nobjects[player];
+		ob = planes[player];
 		inc = 0;
 
 		get_endlevel(ob);
@@ -990,7 +1011,9 @@ void swrestart(void)
 		playmode = PLAYMODE_UNSET;
 	}
 
-	longjmp(envrestart, 0);
+	// The main loop code in swmain.c will detect this and return
+	// to the title screen.
+	restart_flag = true;
 }
 
 // init game
@@ -1025,6 +1048,9 @@ void swinit(int argc, char *argv[])
 			gamenum = starting_level;
 		} else if (!strcasecmp(argv[i], "-q")) {
 			soundflg = 1;
+		} else if (!strcmp(argv[i], "-m") && (i + 1) < argc) {
+			load_custom_level(argv[i + 1]);
+			++i;
 		} else
 #ifdef TCPIP
 		if (!strcasecmp(argv[i], "-l")) {
@@ -1038,6 +1064,12 @@ void swinit(int argc, char *argv[])
 			asynmode = ASYN_CONNECT;
 			++i;
 			snprintf(asynhost, sizeof(asynhost), "%s", argv[i]);
+		} else if (!strcmp(argv[i], "-p")) {
+			if (i + 1 >= argc) {
+				error_exit("insufficient arguments to -p");
+			}
+			++i;
+			asynport = atoi(argv[i]);
 		} else
 #endif
 		{
@@ -1062,18 +1094,14 @@ void swinit(int argc, char *argv[])
 
 	Timer_Init();
 	Vid_Init();
+	Vid_SetVideoPalette(conf_video_palette);
 
 	// dont init speaker if started with -q (quiet)
 	if (soundflg) {
 		Speaker_Init();		// init pc speaker
 	}
 
-	// initgrnd() below needs currgame initialized to set the contents of
-	// the ground[] array for the title screen.
-	currgame = &original_level;
-
 	initsndt();
-	initgrnd();
 	
 	// set playmode if we can, from command line options
 
