@@ -72,15 +72,14 @@ void swmove(void)
 static void nearpln(OBJECTS *ob)
 {
 	OBJECTS *obt, *obc;
-	int i, obx, obclr;
-
-	obt = objtop + 1;
+	int obx, obclr;
 
 	obx = ob->ob_x;
 	obclr = ob->ob_owner->ob_clr;
 
-	for (i = 1; obt->ob_type == PLANE; ++i, ++obt) {
-		if (obclr == obt->ob_owner->ob_clr) {
+	for (obt = objtop; obt != NULL; obt = obt->ob_next) {
+		if (obt->ob_type != PLANE
+		 || obclr == obt->ob_owner->ob_clr) {
 			continue;
 		}
 
@@ -178,8 +177,9 @@ bool moveplyr(OBJECTS *ob)
 		if (endcount <= 0) {
 			if (playmode != PLAYMODE_ASYNCH && !quit) {
 				swrestart();
+				return true;
 			}
-			swend(NULL, YES);
+			swend(NULL, true);
 		}
 	}
 	
@@ -221,10 +221,10 @@ bool moveplyr(OBJECTS *ob)
 			}
 		} else {
 			initplyr(ob);
-			initdisp(YES);
+			initdisp(true);
 			if (endstat == WINNER) {
 				if (ctlbreak()) {
-					swend(NULL, YES);
+					swend(NULL, true);
 				}
 				winner(ob);
 			}
@@ -781,7 +781,7 @@ bool movemiss(OBJECTS *ob)
 			if (obt->ob_missiletarget) {
 				obt = obt->ob_missiletarget;
 			}
-			aim(ob, obt->ob_x, obt->ob_y, NULL, NO);
+			aim(ob, obt->ob_x, obt->ob_y, NULL, false);
 			angle = ob->ob_angle
 			    =
 			    (ob->ob_angle + ob->ob_flaps +
@@ -1076,11 +1076,9 @@ bool moveox(OBJECTS * ob)
 
 
 
-bool crashpln(OBJECTS * obp)
+bool crashpln(OBJECTS *ob)
 {
-	OBJECTS *ob, *obo;
-
-	ob = obp;
+	const original_ob_t *orig_ob = ob->ob_original_ob;
 
 	if (ob->ob_dx < 0) {
 		ob->ob_angle = (ob->ob_angle + 2) % ANGLES;
@@ -1092,9 +1090,8 @@ bool crashpln(OBJECTS * obp)
 	ob->ob_athome = false;
 	ob->ob_dx = ob->ob_dy = ob->ob_ldx = ob->ob_ldy = ob->ob_speed = 0;
 
-	obo = &oobjects[ob->ob_index];
-	ob->ob_hitcount = ((abs(obo->ob_x - ob->ob_x) < SAFERESET)
-	                && (abs(obo->ob_y - ob->ob_y) < SAFERESET))
+	ob->ob_hitcount = ((abs(orig_ob->x - ob->ob_x) < SAFERESET)
+	                && (abs(ob->ob_orig_y - ob->ob_y) < SAFERESET))
 	    ? (MAXCRCOUNT << 1) : MAXCRCOUNT;
 
 	return true;
