@@ -90,7 +90,7 @@ void swsplatbird(void)
 	splat.x = (unsigned) (seed % (SCR_WDTH - 32));
 	splat.y = (unsigned) (seed % (SCR_HGHT - 60)) + 60;
 	splat.sym = symbol_birdsplat;
-	splat.clr = 2;
+	splat.clr = OWNER_PLAYER2;
 
 	add_splat(&splat);
 }
@@ -104,7 +104,7 @@ void swwindshot(void)
 	splat.x = (unsigned) (seed % (SCR_WDTH - 16));
 	splat.y = (unsigned) (seed % (SCR_HGHT - 50)) + 50;
 	splat.sym = symbol_shotwin;
-	splat.clr = 0;
+	splat.clr = OWNER_PLAYER1;
 
 	add_splat(&splat);
 }
