@@ -27,7 +27,8 @@ static void dispribbonrow (int *ribbonid, int ribbons_nr, int y)
 	for (i = 0; i < ribbons_nr; i++) {
 		int ribbon_id = ribbonid[i];
 
-		Vid_DispSymbol(offset, y, symbol_ribbon[ribbon_id], 0);
+		Vid_DispSymbol(offset, y, symbol_ribbon[ribbon_id],
+		               OWNER_PLAYER1);
 		offset += 8;
 	}
 }
@@ -40,10 +41,10 @@ static void dispmedals(OBJECTS *ob)
 	int i;
 
 	for (i = 0; i < ob->ob_score.medals_nr; i++) {
-		int medal_id = ob->ob_score.medalslist[i];
+		int medal_id = ob->ob_score.medals[i];
 
 		Vid_DispSymbol(medal_offset + medal_offsets[medal_id],
-		               11, symbol_medal[medal_id], 0);
+		               11, symbol_medal[medal_id], OWNER_PLAYER1);
 		medal_offset += medal_widths[medal_id];
 	}
 
@@ -97,7 +98,7 @@ static void dispgauges(OBJECTS *ob)
 	dispgge(x += sep, ob->ob_bombs, MAXBOMBS, 2);
 
 	// bullets
- 	dispgge(x += sep, ob->ob_rounds, MAXROUNDS, 3);
+	dispgge(x += sep, ob->ob_rounds, MAXROUNDS, 3);
 
 	if (conf_missiles) {
 
@@ -122,7 +123,8 @@ static void dispmapobjects(void)
 			y = ((ob->ob_y - (ob->ob_newsym->h / 2)) / WRLD_RSY);
 
 			if (y < SCR_MNSH-1) {
-				Vid_PlotPixel(x, y, ob->ob_clr);
+				Vid_PlotPixel(x, y,
+				              Vid_FuselageColor(ob->ob_clr));
 			}
 		}
 	}
