@@ -18,9 +18,11 @@
 
 #include "video.h"
 
+#include "sw.h"
 #include "swasynio.h"
 #include "swconf.h"
 #include "swend.h"
+#include "swgames.h"
 #include "swgrpha.h"
 #include "swinit.h"
 #include "swmain.h"
@@ -66,7 +68,7 @@ void swtitln(void)
 
 	swcolor(3);
 	swposcur(0+X_OFFSET/8, 11);
-	swputs("(c) 2001-2022 Simon Howard, Jesse Smith");
+	swputs("(c) 2001-2024 Simon Howard, Jesse Smith");
 
 	swcolor(3);
 	swposcur(0+X_OFFSET/8, 12);
@@ -76,15 +78,21 @@ void swtitln(void)
 	swcolor(3);
 	swputs(" GPL");
 
+	// We might be playing a custom level, but we need to swap back to
+	// the original level to render the title screen properly.
+	currgame = &original_level;
 	initgrnd();
 	displx = 507-X_OFFSET;
 	swground();
 
-	Vid_DispSymbol(40+X_OFFSET, 180, symbol_plane[0][0], 1);
-	Vid_DispSymbol(130+X_OFFSET, 80, symbol_plane[1][7], 2);
-	Vid_DispSymbol(23+X_OFFSET, ground[530] + 16, symbol_targets[3], 2);
-	Vid_DispSymbol(213+X_OFFSET, ground[720] + 16, symbol_ox[0], 1);
-	Vid_DispSymbol(270+X_OFFSET, 160, symbol_plane_hit[0], 2);
+	Vid_DispSymbol(40+X_OFFSET, 180, symbol_plane[0][0], OWNER_PLAYER1);
+	Vid_DispSymbol(130+X_OFFSET, 80, symbol_plane[1][7], OWNER_PLAYER2);
+	Vid_DispSymbol(23+X_OFFSET, ground[530] + 16, symbol_targets[3],
+	               OWNER_PLAYER2);
+	Vid_DispSymbol(213+X_OFFSET, ground[720] + 16, symbol_ox[0],
+	               OWNER_PLAYER1);
+	Vid_DispSymbol(270+X_OFFSET, 160, symbol_plane_hit[0],
+	               OWNER_PLAYER2);
 
 	for (i = 6, h=165; i; --i, h += 5) {
 		Vid_PlotPixel(280+X_OFFSET, h, 3);
@@ -132,7 +140,7 @@ static bool gethost(void)
 	swputs("Enter Remote Hostname/IP:\n");
 	swgets(asynhost, sizeof(asynhost) - 3);
 
-	return 1;
+	return strcmp(asynhost, "") != 0;
 }
 
 // network menu
@@ -149,7 +157,7 @@ static bool getnet(void)
 		swsndupdate();
 
 		if (ctlbreak()) {
-			swend(NULL, NO);
+			swend(NULL, false);
 		}
 
 		switch (toupper(swgetc() & 0xff)) {
@@ -158,8 +166,7 @@ static bool getnet(void)
 			return 1;
 		case 'C':
 			asynmode = ASYN_CONNECT;
-			gethost();
-			return 1;
+			return gethost();
 		case 27:
 			return 0;
 		}
@@ -179,7 +186,7 @@ static bool getskill(void)
 
 		swsndupdate();
 		if (ctlbreak()) {
-			swend(NULL, NO);
+			swend(NULL, false);
 		}
 		switch (toupper(swgetc() & 0xff)) {
 		case 'N':
@@ -209,11 +216,13 @@ void getgamemode(void)
 		swputs("     N - network game\n");
 #endif
 		swputs("     O - game options\n");
+#ifndef NO_EXIT
 		swputs("     Q - quit game\n");
+#endif
 		Vid_Update();
 
 		if (ctlbreak()) {
-			swend(NULL, NO);
+			swend(NULL, false);
 		}
 
 		c = toupper(swgetc() & 0xff);
@@ -238,9 +247,11 @@ void getgamemode(void)
 			}
 			break;
 #endif
+#ifndef NO_EXIT
 		case 'Q':
 			exit(0);
 			break;
+#endif
 		}
 	}
 }
